datablock ItemData(KeytarItem) {
  category = "Weapon";
  className = "Weapon";

  shapeFile = "./models/keytar_item.dts";
  emap = true;

  uiName = "Keytar";
  iconName = "./icon_keytar";

  image = KeytarImage;
  canDrop = true;

  doColorShift = false;
  colorShiftColor = "0.9 0.1 0.1 1";

  friction = 0.599218;
};

datablock ShapeBaseImageData(KeytarImage) {
  className = "WeaponImage";
  shapeFile = "./models/keytar_image.dts";
  item = KeytarItem;
  emap = true;

  mountPoint = 0;

  offset = "0 0 0";
  eyeOffset = "0 0.3 -2.01";

  // This is used by the instruments mod to determine what sounds to use
  // It's also used to check if an image is even an instrument to begin with
  instrumentType = "Synth";

  doColorShift = true;
  colorShiftColor = KeytarItem.colorShiftColor;

  stateName[0]                     = "Activate";
  stateTimeoutValue[0]             = 0.15;
  stateTransitionOnTimeout[0]       = "Ready";
  stateSound[0]          = weaponSwitchSound;

  stateName[1]                     = "Ready";
  stateTransitionOnTriggerDown[1]  = "Play";
  stateAllowImageChange[1]         = true;
  stateSequence[1]  = "Ready";

  stateName[2]                    = "Play";
  stateTransitionOnTimeout[2]     = "PostPlay";
  stateTimeoutValue[2]            = 0.14;
  stateFire[2]                    = true;
  stateAllowImageChange[2]        = false;
  stateScript[2]                  = "onPlay";
  stateWaitForTimeout[2]      = true;
  stateEmitterTime[2]        = 0.05;

  stateName[3] = "PostPlay";
  stateEmitterTime[3]        = 0.05;
  stateEmitterNode[3]        = "muzzleNode";
  stateTimeoutValue[3]            = 0.01;
  stateTransitionOnTimeout[3]     = "Reload";

  stateName[4]      = "Reload";
  stateSequence[4]                = "Reload";
  stateTransitionOnTriggerUp[4]     = "Ready";
  stateSequence[4]  = "Ready";
};

// ------------------------------------------------------------------------------

function KeytarImage::onMount(%this, %obj, %slot) {
  Parent::onMount(%this, %obj, %slot);

  if (%obj.keytarMode $= "") {
    %obj.keytarMode = %this.instrumentType;
    %obj.setInstrument(%this.instrumentType);
  }

  %obj.playThread(2, armReadyBoth);
  
  %obj.hideNode("RHand");
  %obj.hideNode("RHook");
  %obj.hideNode("LHand");
  %obj.hideNode("LHook");
}

function KeytarImage::onUnmount(%this, %obj, %slot) {
  Parent::onUnmount(%this, %obj, %slot);

  %obj.playThread(2, root);

  if (isObject(%client = %obj.client)) {
    %client.applyBodyParts();
    %client.applyBodyColors();
  }
  else {
    applyDefaultCharacterPrefs(%obj);
  }
}

// ------------------------------------------------------------------------------

function KeytarImage::onPlay(%this, %obj, %pos) {
  if (%obj.keytarMode !$= "") {
    %obj.instrument = %obj.keytarMode;
  }
  else {
    %obj.instrument = %this.instrumentType;
    %obj.keytarMode = %this.instrumentType;

    if (isObject(%client = %obj.client)) {
      messageClient(%client, '', " ");
      messageClient(%client, '', 
        "\c6Type \c3/setKeytarMode \c0instrumentType \c6to set the keytar's instrument mode");
    }
  }
}

// ------------------------------------------------------------------------------

function Player::hasKeytar(%this) {
  %image = %this.getMountedImage(0);
  return isObject(%image) && %image.getID() == KeytarImage.getID();
}

// ------------------------------------------------------------------------------

function serverCmdSetKeytarMode(%client, %instrument, %usingGui) {
  if (!$Pref::Server::Instruments::Enabled) {
    return;
  }

  if (InstrumentsServer.index(%instrument) !$= "") {
    %instrument = _cleanFilename(%instrument);
    %client.keytarMode = %instrument;

    if (isObject(%player = %client.player) && %player.hasKeytar()) {
      %player.instrument = %instrument;
      %player.setInstrument(%instrument);
    }

    if (!%usingGui) {
      messageClient(%client, '', "\c6Keytar set to mode \c3" @ %instrument);
    }
  }
  else if (!%usingGui) {
    messageClient(%client, '', "Instrument '" @ %instrument @ "' not found");
  }
}

function serverCmdSKM(%client, %instrument) {
  serverCmdSetKeytarMode(%client, %instrument);
}

function serverCmdGetInstrumentType(%client) {
  if (!isObject(%player = %client.player)) {
    return;
  }

  if (isObject(%instrument = %player.getMountedImage(0))) {
    if (%instrument.instrumentType !$= "") {
      messageClient(%client, '', "\c3Selected instrument type: \c6" @ %instrument.instrumentType);
    }
  }
}

// ------------------------------------------------------------------------------

package Instrument_Keytar__server {
  function Player::setInstrument(%this, %instrument) {
    if (%instrument !$= "") {
      if (isObject(%client = %this.client)) {
        %keytarMode = %client.keytarMode;
      }
      else {
        %keytarMode = %this.keytarMode;
      }

      if (%this.hasKeytar() && %keytarMode !$= "") {
        %instrument = %keytarMode;
      }
    }
    
    Parent::setInstrument(%this, %instrument);
  }

  function serverCmdOnSelectInstrument(%client, %instrument) {
    Parent::serverCmdOnSelectInstrument(%client, %instrument);

    if (isObject(%player = %client.player) && %player.hasKeytar()) {
      serverCmdSetKeytarMode(%client, %Instrument, 1);
      commandToClient(%client, 'setInstrument', %instrument);
    }
  }
};

activatePackage(Instrument_Keytar__server);
